//
//  utils.swift.swift
//  face_plugin
//
//  Created by 胡晓庆 on 2022/3/21.
//

import Foundation
import AVFoundation
import UIKit

extension DispatchQueue {
    /// SwifterSwift: A Boolean value indicating whether the current dispatch queue is the main queue.
    static var isMainQueue: Bool {
        enum Static {
            static var key: DispatchSpecificKey<Void> = {
                let key = DispatchSpecificKey<Void>()
                DispatchQueue.main.setSpecific(key: key, value: ())
                return key
            }()
        }
        return DispatchQueue.getSpecific(key: Static.key) != nil
    }
}

func dispatchMainAsyncSafe(execute work: @escaping () -> Void) {
    if DispatchQueue.isMainQueue {
        work()
    } else {
        DispatchQueue.main.async(execute: work)
    }
}

func DLog<T>(_ message: T, file: String = #file, methodName: String = #function, lineNumber: Int = #line) {
    #if DEBUG
    let fileName = (file as NSString).lastPathComponent
    
    let nowDate = Date()
    let format = DateFormatter()
    format.dateFormat = "[HH:mm:ss.SSSSSS]"
    let dateString = format.string(from: nowDate)
    
    print("\(dateString) \(fileName) \(methodName) line:\(lineNumber) --\(message)")
    #endif
}

extension UIImage {
    
    class func imageFromSampleBuffer(sampleBuffer: CMSampleBuffer, videoOrientation: AVCaptureVideoOrientation) -> UIImage? {
        if let imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer) {
            let context = CIContext()
            var ciImage = CIImage(cvPixelBuffer: imageBuffer)

            // FIXME: - change to Switch
            if videoOrientation == .landscapeLeft {
                ciImage = ciImage.oriented(forExifOrientation: 3)
            } else if videoOrientation == .landscapeRight {
                ciImage = ciImage.oriented(forExifOrientation: 1)
            } else if videoOrientation == .portrait {
                ciImage = ciImage.oriented(forExifOrientation: 6)
            } else if videoOrientation == .portraitUpsideDown {
                ciImage = ciImage.oriented(forExifOrientation: 8)
            }

            if let cgImage = context.createCGImage(ciImage, from: ciImage.extent) {
                return UIImage(cgImage: cgImage)
            }
        }
        return nil
    }
}


func currentViewController() -> UIViewController? {
    
    var currentVC: UIViewController?
    var nextVC: UIViewController? = UIApplication.shared.keyWindow?.rootViewController
    
    while nextVC != nil {
        if let tab: UITabBarController = nextVC as? UITabBarController {
            currentVC = tab
            nextVC = tab.selectedViewController
            continue
        }
        if let nav: UINavigationController = nextVC as? UINavigationController {
            currentVC = nav
            nextVC = nav.topViewController
            continue
        }
        if let vc: UIViewController = nextVC {
            currentVC = vc
            nextVC = vc.presentedViewController
        }
    }
    
    return currentVC
    
}
